﻿
namespace Yuuna.Contracts.Artisan.Cascade
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.Immutable;

    internal sealed class Session : IInternalSession
    {
        private readonly Queue<object> _q;
        private readonly object _lockObject;
        internal Session()
        {
            this._lockObject = new object();
            this._q = new Queue<object>();
        }

        #region IInternalEventDataBus 

        private event Action<Queue<object>> _completed;

        event Action<Queue<object>> IInternalEventDataBus.Completed
        {
            add => this._completed += value;
            remove => this._completed -= value;
        }

        void IInternalEventDataBus.OnComplete() => this._completed?.Invoke(this._q);
        #endregion

        #region IInternalStorage 
        void IInternalStorage.Store<T>(T data)
        {
            lock (this._lockObject)
                this._q.Enqueue(data);
        }
        #endregion

        #region IReadOnlyList<object>
         

        public int Count
        {
            get
            {
                lock (this._lockObject)
                    return this._q.Count;
            }
        }

        public IEnumerator<object> GetEnumerator() => this._q.ToImmutableList().GetEnumerator();
        IEnumerator IEnumerable.GetEnumerator() => this.GetEnumerator();

        #endregion

    }
}
