﻿
namespace Yuuna.Contracts.Interaction
{
    using System;
    using System.ComponentModel;

    /// <summary>
    /// 回應物件，表示插件回覆的資訊。
    /// </summary>
    public sealed class Response
    {
        /// <summary>
        /// 情緒。
        /// </summary>
        public Mood Mood { get; }
        /// <summary>
        /// 訊息。
        /// </summary>
        public string Message { get; }


        /// <summary>
        /// 建立新回應物件實體。
        /// </summary>
        /// <param name="mood">情緒。</param>
        /// <param name="message">訊息。</param>
        public Response(Mood mood, string message)
        {
            this.Mood = mood;
            this.Message = message ?? string.Empty;
        }
        public static implicit operator Response(ValueTuple<Mood, string> full)
        {
            return new Response(full.Item1, full.Item2);
        }
        public static implicit operator Response(string message)
        {
            return new Response(Moods.Normal, message);
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public void Deconstruct(out Mood mood, out string message)
        {
            mood = this.Mood;
            message = this.Message;
        }

        public override string ToString() => this.Message;
    }
}
