﻿
namespace Yuuna.Contracts.Optimization
{
    using System;
    using System.Collections.Generic;
    using System.Collections.Immutable;
    using System.Diagnostics;
    using System.Diagnostics.CodeAnalysis;
    using System.Linq;
    using System.Runtime.CompilerServices;
    using System.Text;
    using System.Threading.Tasks;
    using Yuuna.Contracts.Patterns;
    using Yuuna.Contracts.Plugins;

    public abstract class StrategyBase
    {
        protected StrategyBase()
        { 
        }

        public abstract string Name { get; }

        protected abstract IComparer<IScore> Comparer { get; }


        internal IImmutableList<IScore> Assess(PluginBase plugin, IImmutableList<string> words, IPatternSet set)
        {
            var scores = new List<IScore>();
            var immutable = set.ToImmutable();
            foreach (var pattern in immutable)
            {
                var score = this.Evaluate(plugin,words, pattern);
                if (score.SequentialMatches.Count > 0) // 過濾掉符合為 0 的文法
                    scores.Add(score);
            }
            scores.Sort(this.Comparer);

            return scores.ToImmutableArray();
        }

        /// <summary>
        /// 評估
        /// </summary>
        /// <param name="words"></param>
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        private IScore Evaluate(PluginBase plugin ,IImmutableList<string> words, IPattern pattern)
        {
            var matches = ImmutableArray.CreateBuilder<IPair>();

            using (var iterator = pattern.ToImmutable().GetEnumerator())
            {
                iterator.MoveNext();
                foreach (var word in words)
                {
                    if (iterator.Current.TryGetSynonym(word, out var s))
                    {
                        matches.Add(new Pair(s.Owner.Key, s));
                        if (!iterator.MoveNext())
                            break;
                    } 
                }
            }

            return new Score(plugin, matches.ToImmutable(), pattern);
        }
    }


}