﻿
namespace Yuuna.Contracts.Plugins
{
    using System;
    using System.Collections.Generic;
    using System.Collections.Immutable;
    using System.Runtime.CompilerServices;
    using Yuuna.Contracts.Interaction;
    using Yuuna.Contracts.Optimization;
    using Yuuna.Contracts.Patterns;
    using Yuuna.Contracts.Artisan.Cascade;
    using Yuuna.Contracts.Semantics;
    using Yuuna.Contracts.TextSegmention;
     
    public abstract class PluginBase
    {
        private readonly PatternFactory _patternfactory;

        public PluginBase()
        {
            this._patternfactory = new PatternFactory();
        }

        /// <summary>
        /// 在初始化前引發。
        /// </summary>
        /// <param name="segmenter"></param>
        protected virtual void BeforeInitialize(ITextSegmenter segmenter)
        { 
        }

        internal bool Initialize(ITextSegmenter textSegmenter)
        {
            try
            {
                this.BeforeInitialize(textSegmenter);

                var groupManager = new GroupManager();
                this.BuildPatterns(groupManager, this._patternfactory );
                textSegmenter.Load(groupManager);
                this.AfterInitialize();
                return true;
            }
            catch(Exception e)
            {
                Console.WriteLine(e);
                return false;
            }
        }

        /// <summary>
        /// 在初始化後引發。
        /// </summary>
        protected virtual void AfterInitialize()
        {
        }

        /// <summary>
        /// 建立模式規則。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="p"></param>
        protected abstract void BuildPatterns(IGroupManager g, IPatternBuilder p);

        internal IImmutableList<IScore> Evaluate(StrategyBase strategy, IImmutableList<string> cutted)
        { 
            return strategy.Assess(this, cutted, this._patternfactory);
        }

        internal Invoke SelectBest(IScore score)
        {
            return this._patternfactory.TryGet(score.Contrast, out var b) ? b : null;
        }
    }


}
