﻿
namespace Yuuna.Contracts.Plugins
{
    using System;
    using System.Collections.Generic;
    using System.Collections.Immutable;
    using System.Linq;
    using System.Text;
    using Yuuna.Contracts.Optimization;
    using Yuuna.Contracts.Artisan.Cascade;
    using Yuuna.Contracts.Patterns;
    using Yuuna.Contracts.TextSegmention;
    using Yuuna.Contracts.Utilities;

    public class PluginGateway
    { 
        public static void Main(string[] args)
        { 
            // Send("打開門");
            // Send("開燈");
              Send("打開燈"); 
        }

        public static void Send(string text)
        {
            ITextSegmenter segmenter = new JiebaTextSegmenter();
            var allPlugins = new PluginBase[] { new Fake() };
            foreach (var item in allPlugins)
            {
                item.Initialize(segmenter);
                Console.WriteLine("已載入模組: "+item.GetType().AssemblyQualifiedName);
            }

            Console.WriteLine("我: " + text);
            var cutted = segmenter.Cut(text);
            Console.WriteLine($"來自分詞器 {segmenter.Name} 的分詞結果: [ {string.Join(", ",cutted)} ]");
            var strategy = new DefaultStrategy();
            var list = new List<IScore>();
            foreach (var p in allPlugins) 
                list.AddRange(p.Evaluate(strategy, cutted));
            list.Sort(ScoreComparer.Default);

            var sb = new StringBuilder("Bot: ");
            var best = list.LastOrDefault();
            if (best is null)
                sb.Append("我不懂你的意思");
            else if (best.IsCompacted)
            {
                var resp = best.Plugin.SelectBest(best).Invoke(best);
                Console.WriteLine(   resp.Mood.ToString());
                sb.Append(resp.Message);
            }
            else
            {
                var choices = list.Where(s => s.Missing.Count <= 1).ToImmutableArray();
                switch (choices.Length)
                {
                    case 1:
                        {
                            Console.WriteLine("#1");
                            sb.Append("你" + new[] { "是想", "想要" }.RandomTakeOne() + " ");
                            sb.Append(choices.Select(x => x.Contrast.ToImmutable().Aggregate(string.Empty, (s, g) => s += g.ToImmutable().RandomTakeOne().ToImmutable().RandomTakeOne())));
                            sb.Append(" 嗎?");
                        }
                        break;


                    case 2:
                        {
                            Console.WriteLine("#2");
                            sb.Append("你" + new[] { "是想", "想要" }.RandomTakeOne() + " ");
                            sb.AppendJoin(" 還是 ", choices.Select(x => x.Contrast.ToImmutable().Aggregate(string.Empty, (s, g) => s += g.ToImmutable().RandomTakeOne().ToImmutable().RandomTakeOne())));
                            sb.Append(" ?");
                        }
                        break;


                    default:
                        Console.WriteLine("#n");
                        sb.Append("我不太清楚你想做什麼");
                        break;
                }
            }

            Console.WriteLine(sb);

        }
    }

}
